/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.4.0 (build: 2013-06-20 19:08:55 UTC)
 *  on 2013-06-25 at 00:10:01 UTC 
 */

package com.google.api.services.customsearch;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for Customsearch (v1).
 *
 * <p>
 * Lets you search over a website or collection of websites
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/custom-search/v1/using_rest" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link CustomsearchRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Customsearch extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the CustomSearch API library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "customsearch/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Customsearch(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  Customsearch(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Cse collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Customsearch customsearch = new Customsearch(...);}
   *   {@code Customsearch.Cse.List request = customsearch.cse().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Cse cse() {
    return new Cse();
  }

  /**
   * The "cse" collection of methods.
   */
  public class Cse {

    /**
     * Returns metadata about the search performed, metadata about the custom search engine used for the
     * search, and the search results.
     *
     * Create a request for the method "cse.list".
     *
     * This request holds the parameters needed by the the customsearch server.  After setting any
     * optional parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param q Query
     * @return the request
     */
    public List list(String q) throws java.io.IOException {
      List result = new List(q);
      initialize(result);
      return result;
    }

    public class List extends CustomsearchRequest<com.google.api.services.customsearch.model.Search> {

      private static final String REST_PATH = "v1";

      /**
       * Returns metadata about the search performed, metadata about the custom search engine used for
       * the search, and the search results.
       *
       * Create a request for the method "cse.list".
       *
       * This request holds the parameters needed by the the customsearch server.  After setting any
       * optional parameters, call the {@link List#execute()} method to invoke the remote operation. <p>
       * {@link List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param q Query
       * @since 1.13
       */
      protected List(String q) {
        super(Customsearch.this, "GET", REST_PATH, null, com.google.api.services.customsearch.model.Search.class);
        this.q = Preconditions.checkNotNull(q, "Required parameter q must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Query */
      @com.google.api.client.util.Key
      private String q;

      /** Query
       */
      public String getQ() {
        return q;
      }

      /** Query */
      public List setQ(String q) {
        this.q = q;
        return this;
      }

      /** The sort expression to apply to the results */
      @com.google.api.client.util.Key
      private String sort;

      /** The sort expression to apply to the results
       */
      public String getSort() {
        return sort;
      }

      /** The sort expression to apply to the results */
      public List setSort(String sort) {
        this.sort = sort;
        return this;
      }

      /**
     * Provides additional search terms to check for in a document, where each document in the
     * search results must contain at least one of the additional search terms
     */
      @com.google.api.client.util.Key
      private String orTerms;

      /** Provides additional search terms to check for in a document, where each document in the search
     results must contain at least one of the additional search terms
       */
      public String getOrTerms() {
        return orTerms;
      }

      /**
     * Provides additional search terms to check for in a document, where each document in the
     * search results must contain at least one of the additional search terms
     */
      public List setOrTerms(String orTerms) {
        this.orTerms = orTerms;
        return this;
      }

      /** Creates a range in form as_nlo value..as_nhi value and attempts to append it to query */
      @com.google.api.client.util.Key
      private String highRange;

      /** Creates a range in form as_nlo value..as_nhi value and attempts to append it to query
       */
      public String getHighRange() {
        return highRange;
      }

      /** Creates a range in form as_nlo value..as_nhi value and attempts to append it to query */
      public List setHighRange(String highRange) {
        this.highRange = highRange;
        return this;
      }

      /** Number of search results to return */
      @com.google.api.client.util.Key
      private Long num;

      /** Number of search results to return [default: 10]
       */
      public Long getNum() {
        return num;
      }

      /** Number of search results to return */
      public List setNum(Long num) {
        this.num = num;
        return this;
      }

      /** Country restrict(s). */
      @com.google.api.client.util.Key
      private String cr;

      /** Country restrict(s).
       */
      public String getCr() {
        return cr;
      }

      /** Country restrict(s). */
      public List setCr(String cr) {
        this.cr = cr;
        return this;
      }

      /** Returns images of a type, which can be one of: clipart, face, lineart, news, and photo. */
      @com.google.api.client.util.Key
      private String imgType;

      /** Returns images of a type, which can be one of: clipart, face, lineart, news, and photo.
       */
      public String getImgType() {
        return imgType;
      }

      /** Returns images of a type, which can be one of: clipart, face, lineart, news, and photo. */
      public List setImgType(String imgType) {
        this.imgType = imgType;
        return this;
      }

      /** Geolocation of end user. */
      @com.google.api.client.util.Key
      private String gl;

      /** Geolocation of end user.
       */
      public String getGl() {
        return gl;
      }

      /** Geolocation of end user. */
      public List setGl(String gl) {
        this.gl = gl;
        return this;
      }

      /** Specifies that all search results should be pages that are related to the specified URL */
      @com.google.api.client.util.Key
      private String relatedSite;

      /** Specifies that all search results should be pages that are related to the specified URL
       */
      public String getRelatedSite() {
        return relatedSite;
      }

      /** Specifies that all search results should be pages that are related to the specified URL */
      public List setRelatedSite(String relatedSite) {
        this.relatedSite = relatedSite;
        return this;
      }

      /** Specifies the search type: image. */
      @com.google.api.client.util.Key
      private String searchType;

      /** Specifies the search type: image.
       */
      public String getSearchType() {
        return searchType;
      }

      /** Specifies the search type: image. */
      public List setSearchType(String searchType) {
        this.searchType = searchType;
        return this;
      }

      /**
     * Returns images of a specified type. Some of the allowed values are: bmp, gif, png, jpg, svg,
     * pdf, ...
     */
      @com.google.api.client.util.Key
      private String fileType;

      /** Returns images of a specified type. Some of the allowed values are: bmp, gif, png, jpg, svg, pdf,
     ...
       */
      public String getFileType() {
        return fileType;
      }

      /**
     * Returns images of a specified type. Some of the allowed values are: bmp, gif, png, jpg, svg,
     * pdf, ...
     */
      public List setFileType(String fileType) {
        this.fileType = fileType;
        return this;
      }

      /** The index of the first result to return */
      @com.google.api.client.util.Key
      private Long start;

      /** The index of the first result to return
       */
      public Long getStart() {
        return start;
      }

      /** The index of the first result to return */
      public List setStart(Long start) {
        this.start = start;
        return this;
      }

      /**
     * Returns images of a specific dominant color: yellow, green, teal, blue, purple, pink, white,
     * gray, black and brown.
     */
      @com.google.api.client.util.Key
      private String imgDominantColor;

      /** Returns images of a specific dominant color: yellow, green, teal, blue, purple, pink, white, gray,
     black and brown.
       */
      public String getImgDominantColor() {
        return imgDominantColor;
      }

      /**
     * Returns images of a specific dominant color: yellow, green, teal, blue, purple, pink, white,
     * gray, black and brown.
     */
      public List setImgDominantColor(String imgDominantColor) {
        this.imgDominantColor = imgDominantColor;
        return this;
      }

      /** The language restriction for the search results */
      @com.google.api.client.util.Key
      private String lr;

      /** The language restriction for the search results
       */
      public String getLr() {
        return lr;
      }

      /** The language restriction for the search results */
      public List setLr(String lr) {
        this.lr = lr;
        return this;
      }

      /** Specifies all search results should be pages from a given site */
      @com.google.api.client.util.Key
      private String siteSearch;

      /** Specifies all search results should be pages from a given site
       */
      public String getSiteSearch() {
        return siteSearch;
      }

      /** Specifies all search results should be pages from a given site */
      public List setSiteSearch(String siteSearch) {
        this.siteSearch = siteSearch;
        return this;
      }

      /** The URL of a linked custom search engine */
      @com.google.api.client.util.Key
      private String cref;

      /** The URL of a linked custom search engine
       */
      public String getCref() {
        return cref;
      }

      /** The URL of a linked custom search engine */
      public List setCref(String cref) {
        this.cref = cref;
        return this;
      }

      /** Specifies all search results are from a time period */
      @com.google.api.client.util.Key
      private String dateRestrict;

      /** Specifies all search results are from a time period
       */
      public String getDateRestrict() {
        return dateRestrict;
      }

      /** Specifies all search results are from a time period */
      public List setDateRestrict(String dateRestrict) {
        this.dateRestrict = dateRestrict;
        return this;
      }

      /** Search safety level */
      @com.google.api.client.util.Key
      private String safe;

      /** Search safety level [default: off]
       */
      public String getSafe() {
        return safe;
      }

      /** Search safety level */
      public List setSafe(String safe) {
        this.safe = safe;
        return this;
      }

      /** Turns off the translation between zh-CN and zh-TW. */
      @com.google.api.client.util.Key
      private String c2coff;

      /** Turns off the translation between zh-CN and zh-TW.
       */
      public String getC2coff() {
        return c2coff;
      }

      /** Turns off the translation between zh-CN and zh-TW. */
      public List setC2coff(String c2coff) {
        this.c2coff = c2coff;
        return this;
      }

      /** The local Google domain to use to perform the search. */
      @com.google.api.client.util.Key
      private String googlehost;

      /** The local Google domain to use to perform the search.
       */
      public String getGooglehost() {
        return googlehost;
      }

      /** The local Google domain to use to perform the search. */
      public List setGooglehost(String googlehost) {
        this.googlehost = googlehost;
        return this;
      }

      /** Appends the extra query terms to the query. */
      @com.google.api.client.util.Key
      private String hq;

      /** Appends the extra query terms to the query.
       */
      public String getHq() {
        return hq;
      }

      /** Appends the extra query terms to the query. */
      public List setHq(String hq) {
        this.hq = hq;
        return this;
      }

      /** Identifies a phrase that all documents in the search results must contain */
      @com.google.api.client.util.Key
      private String exactTerms;

      /** Identifies a phrase that all documents in the search results must contain
       */
      public String getExactTerms() {
        return exactTerms;
      }

      /** Identifies a phrase that all documents in the search results must contain */
      public List setExactTerms(String exactTerms) {
        this.exactTerms = exactTerms;
        return this;
      }

      /** Sets the user interface language. */
      @com.google.api.client.util.Key
      private String hl;

      /** Sets the user interface language.
       */
      public String getHl() {
        return hl;
      }

      /** Sets the user interface language. */
      public List setHl(String hl) {
        this.hl = hl;
        return this;
      }

      /** Creates a range in form as_nlo value..as_nhi value and attempts to append it to query */
      @com.google.api.client.util.Key
      private String lowRange;

      /** Creates a range in form as_nlo value..as_nhi value and attempts to append it to query
       */
      public String getLowRange() {
        return lowRange;
      }

      /** Creates a range in form as_nlo value..as_nhi value and attempts to append it to query */
      public List setLowRange(String lowRange) {
        this.lowRange = lowRange;
        return this;
      }

      /**
     * Returns images of a specified size, where size can be one of: icon, small, medium, large,
     * xlarge, xxlarge, and huge.
     */
      @com.google.api.client.util.Key
      private String imgSize;

      /** Returns images of a specified size, where size can be one of: icon, small, medium, large, xlarge,
     xxlarge, and huge.
       */
      public String getImgSize() {
        return imgSize;
      }

      /**
     * Returns images of a specified size, where size can be one of: icon, small, medium, large,
     * xlarge, xxlarge, and huge.
     */
      public List setImgSize(String imgSize) {
        this.imgSize = imgSize;
        return this;
      }

      /** Returns black and white, grayscale, or color images: mono, gray, and color. */
      @com.google.api.client.util.Key
      private String imgColorType;

      /** Returns black and white, grayscale, or color images: mono, gray, and color.
       */
      public String getImgColorType() {
        return imgColorType;
      }

      /** Returns black and white, grayscale, or color images: mono, gray, and color. */
      public List setImgColorType(String imgColorType) {
        this.imgColorType = imgColorType;
        return this;
      }

      /**
     * Filters based on licensing. Supported values include: cc_publicdomain, cc_attribute,
     * cc_sharealike, cc_noncommercial, cc_nonderived and combinations of these.
     */
      @com.google.api.client.util.Key
      private String rights;

      /** Filters based on licensing. Supported values include: cc_publicdomain, cc_attribute, cc_sharealike,
     cc_noncommercial, cc_nonderived and combinations of these.
       */
      public String getRights() {
        return rights;
      }

      /**
     * Filters based on licensing. Supported values include: cc_publicdomain, cc_attribute,
     * cc_sharealike, cc_noncommercial, cc_nonderived and combinations of these.
     */
      public List setRights(String rights) {
        this.rights = rights;
        return this;
      }

      /**
     * Identifies a word or phrase that should not appear in any documents in the search results
     */
      @com.google.api.client.util.Key
      private String excludeTerms;

      /** Identifies a word or phrase that should not appear in any documents in the search results
       */
      public String getExcludeTerms() {
        return excludeTerms;
      }

      /**
     * Identifies a word or phrase that should not appear in any documents in the search results
     */
      public List setExcludeTerms(String excludeTerms) {
        this.excludeTerms = excludeTerms;
        return this;
      }

      /** Controls turning on or off the duplicate content filter. */
      @com.google.api.client.util.Key
      private String filter;

      /** Controls turning on or off the duplicate content filter.
       */
      public String getFilter() {
        return filter;
      }

      /** Controls turning on or off the duplicate content filter. */
      public List setFilter(String filter) {
        this.filter = filter;
        return this;
      }

      /** Specifies that all search results should contain a link to a particular URL */
      @com.google.api.client.util.Key
      private String linkSite;

      /** Specifies that all search results should contain a link to a particular URL
       */
      public String getLinkSite() {
        return linkSite;
      }

      /** Specifies that all search results should contain a link to a particular URL */
      public List setLinkSite(String linkSite) {
        this.linkSite = linkSite;
        return this;
      }

      /** The custom search engine ID to scope this search query */
      @com.google.api.client.util.Key
      private String cx;

      /** The custom search engine ID to scope this search query
       */
      public String getCx() {
        return cx;
      }

      /** The custom search engine ID to scope this search query */
      public List setCx(String cx) {
        this.cx = cx;
        return this;
      }

      /**
     * Controls whether to include or exclude results from the site named in the as_sitesearch
     * parameter
     */
      @com.google.api.client.util.Key
      private String siteSearchFilter;

      /** Controls whether to include or exclude results from the site named in the as_sitesearch parameter
       */
      public String getSiteSearchFilter() {
        return siteSearchFilter;
      }

      /**
     * Controls whether to include or exclude results from the site named in the as_sitesearch
     * parameter
     */
      public List setSiteSearchFilter(String siteSearchFilter) {
        this.siteSearchFilter = siteSearchFilter;
        return this;
      }

    }

  }

  /**
   * Builder for {@link Customsearch}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Customsearch}. */
    @Override
    public Customsearch build() {
      return new Customsearch(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link CustomsearchRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setCustomsearchRequestInitializer(
        CustomsearchRequestInitializer customsearchRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(customsearchRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
